"""
Generated class for ValueSet. 
Time: 2024-06-14 18:37:49
"""

from fhirmodels.R4B.BackboneElement import *
from fhirmodels.R4B.CodeableConcept import *
from fhirmodels.R4B.Coding import *
from fhirmodels.R4B.ContactDetail import *
from fhirmodels.R4B.DomainResource import *
from fhirmodels.R4B.Extension import *
from fhirmodels.R4B.Identifier import *
from fhirmodels.R4B.Meta import *
from fhirmodels.R4B.Narrative import *
from fhirmodels.R4B.Resource import *
from fhirmodels.R4B.UsageContext import *


class Designation(FhirBaseModel):
    """Additional representations for this concept when used in this value set - other languages, aliases, specialized purposes, used for particular purposes, etc.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param str language: Human language of the designation
    :param Coding use: Types of uses of designations
    :param str value: The text value for this designation
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "use": {"class_name": "Coding", "is_contained": False},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        language: "str" = None,
        use: "Coding" = None,
        value: "str" = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.language = language
        self.use = use
        self.value = value

    @classmethod
    def from_dict(cls, data: dict) -> "ValueSet":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "ValueSet":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class Concept(FhirBaseModel):
    """Specifies a concept to be included or excluded.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param str code: Code or expression from system
    :param str display: Text to display for this code for this value set in this valueset
    :param Designation designation: Additional representations for this concept
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "designation": {"class_name": "Designation", "is_contained": True},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        code: "str" = None,
        display: "str" = None,
        designation: list["Designation"] = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.code = code
        self.display = display
        self.designation = designation or []

    @classmethod
    def from_dict(cls, data: dict) -> "ValueSet":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "ValueSet":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class Filter(FhirBaseModel):
    """Select concepts by specify a matching criterion based on the properties (including relationships) defined by the system, or on filters defined by the system. If multiple filters are specified, they SHALL all be true.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param str property: A property/filter defined by the code system
    :param str op: = | is-a | descendent-of | is-not-a | regex | in | not-in | generalizes | exists
    :param str value: Code from the system, or regex criteria, or boolean value for exists
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        property: "str" = None,
        op: "str" = None,
        value: "str" = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.property = property
        self.op = op
        self.value = value

    @classmethod
    def from_dict(cls, data: dict) -> "ValueSet":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "ValueSet":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class Include(FhirBaseModel):
    """Include one or more codes from a code system or other value set(s).:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param str system: The system the codes come from
    :param str version: Specific version of the code system referred to
    :param Concept concept: A concept defined in the system
    :param Filter filter: Select codes/concepts by their properties (including relationships)
    :param str valueSet: Select the contents included in this value set
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "concept": {"class_name": "Concept", "is_contained": True},
        "filter": {"class_name": "Filter", "is_contained": True},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        system: "str" = None,
        version: "str" = None,
        concept: list["Concept"] = None,
        filter: list["Filter"] = None,
        valueSet: list["str"] = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.system = system
        self.version = version
        self.concept = concept or []
        self.filter = filter or []
        self.valueSet = valueSet or []

    @classmethod
    def from_dict(cls, data: dict) -> "ValueSet":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "ValueSet":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class Compose(FhirBaseModel):
    """A set of criteria that define the contents of the value set by including or excluding codes selected from the specified code system(s) that the value set draws from. This is also known as the Content Logical Definition (CLD).:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param str lockedDate: Fixed date for references with no specified version (transitive)
    :param bool inactive: Whether inactive codes are in the value set
    :param Include include: Include one or more codes from a code system or other value set(s)
    :param Exclude exclude: Explicitly exclude codes from a code system or other value sets
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "include": {"class_name": "Include", "is_contained": True},
        "exclude": {"class_name": "Exclude", "is_contained": True},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        lockedDate: "str" = None,
        inactive: "bool" = None,
        include: list["Include"] = None,
        exclude: list["Exclude"] = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.lockedDate = lockedDate
        self.inactive = inactive
        self.include = include or []
        self.exclude = exclude or []

    @classmethod
    def from_dict(cls, data: dict) -> "ValueSet":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "ValueSet":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class Parameter(FhirBaseModel):
    """A parameter that controlled the expansion process. These parameters may be used by users of expanded value sets to check whether the expansion is suitable for a particular purpose, or to pick the correct expansion.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param str name: Name as assigned by the client or server
    :param str valueString: Value of the named parameter
    :param bool valueBoolean: Value of the named parameter
    :param int valueInteger: Value of the named parameter
    :param float valueDecimal: Value of the named parameter
    :param str valueUri: Value of the named parameter
    :param str valueCode: Value of the named parameter
    :param str valueDateTime: Value of the named parameter
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        name: "str" = None,
        valueString: "str" = None,
        valueBoolean: "bool" = None,
        valueInteger: "int" = None,
        valueDecimal: "float" = None,
        valueUri: "str" = None,
        valueCode: "str" = None,
        valueDateTime: "str" = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.name = name
        self.valueString = valueString
        self.valueBoolean = valueBoolean
        self.valueInteger = valueInteger
        self.valueDecimal = valueDecimal
        self.valueUri = valueUri
        self.valueCode = valueCode
        self.valueDateTime = valueDateTime

    @classmethod
    def from_dict(cls, data: dict) -> "ValueSet":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "ValueSet":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class Contains(FhirBaseModel):
    """The codes that are contained in the value set expansion.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param str system: System value for the code
    :param bool abstract: If user cannot select this entry
    :param bool inactive: If concept is inactive in the code system
    :param str version: Version in which this code/display is defined
    :param str code: Code - if blank, this is not a selectable code
    :param str display: User display for the concept
    :param Designation designation: Additional representations for this item
    :param Contains contains: Codes contained under this entry
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "designation": {"class_name": "Designation", "is_contained": True},
        "contains": {"class_name": "Contains", "is_contained": True},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        system: "str" = None,
        abstract: "bool" = None,
        inactive: "bool" = None,
        version: "str" = None,
        code: "str" = None,
        display: "str" = None,
        designation: list["Designation"] = None,
        contains: list["Contains"] = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.system = system
        self.abstract = abstract
        self.inactive = inactive
        self.version = version
        self.code = code
        self.display = display
        self.designation = designation or []
        self.contains = contains or []

    @classmethod
    def from_dict(cls, data: dict) -> "ValueSet":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "ValueSet":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class Expansion(FhirBaseModel):
    """A value set can also be "expanded", where the value set is turned into a simple collection of enumerated codes. This element holds the expansion, if it has been performed.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param str identifier: Identifies the value set expansion (business identifier)
    :param str timestamp: Time ValueSet expansion happened
    :param int total: Total number of codes in the expansion
    :param int offset: Offset at which this resource starts
    :param Parameter parameter: Parameter that controlled the expansion process
    :param Contains contains: Codes in the value set
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "parameter": {"class_name": "Parameter", "is_contained": True},
        "contains": {"class_name": "Contains", "is_contained": True},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        identifier: "str" = None,
        timestamp: "str" = None,
        total: "int" = None,
        offset: "int" = None,
        parameter: list["Parameter"] = None,
        contains: list["Contains"] = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.identifier = identifier
        self.timestamp = timestamp
        self.total = total
        self.offset = offset
        self.parameter = parameter or []
        self.contains = contains or []

    @classmethod
    def from_dict(cls, data: dict) -> "ValueSet":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "ValueSet":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class ValueSet(DomainResource):
    """A ValueSet resource instance specifies a set of codes drawn from one or more code systems, intended for use in a particular context. Value sets link between [[[CodeSystem]]] definitions and their use in [coded elements](terminologies.html).
    :param str id: Logical id of this artifact
    :param Meta meta: Metadata about the resource
    :param str implicitRules: A set of rules under which this content was created
    :param str language: Language of the resource content
    :param Narrative text: Text summary of the resource, for human interpretation
    :param Resource contained: Contained, inline Resources
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored
    :param str url: Canonical identifier for this value set, represented as a URI (globally unique)
    :param Identifier identifier: Additional identifier for the value set (business identifier)
    :param str version: Business version of the value set
    :param str name: Name for this value set (computer friendly)
    :param str title: Name for this value set (human friendly)
    :param str status: draft | active | retired | unknown
    :param bool experimental: For testing purposes, not real usage
    :param str date: Date last changed
    :param str publisher: Name of the publisher (organization or individual)
    :param ContactDetail contact: Contact details for the publisher
    :param str description: Natural language description of the value set
    :param UsageContext useContext: The context that the content is intended to support
    :param CodeableConcept jurisdiction: Intended jurisdiction for value set (if applicable)
    :param bool immutable: Indicates whether or not any change to the content logical definition may occur
    :param str purpose: Why this value set is defined
    :param str copyright: Use and/or publishing restrictions
    :param Compose compose: Content logical definition of the value set (CLD)
    :param Expansion expansion: Used when the value set is "expanded"
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "meta": {"class_name": "Meta", "is_contained": False},
        "text": {"class_name": "Narrative", "is_contained": False},
        "contained": {"class_name": "Resource", "is_contained": False},
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "identifier": {"class_name": "Identifier", "is_contained": False},
        "contact": {"class_name": "ContactDetail", "is_contained": False},
        "useContext": {"class_name": "UsageContext", "is_contained": False},
        "jurisdiction": {"class_name": "CodeableConcept", "is_contained": False},
        "compose": {"class_name": "Compose", "is_contained": True},
        "expansion": {"class_name": "Expansion", "is_contained": True},
    }

    def __init__(
        self,
        id: "str" = None,
        meta: "Meta" = None,
        implicitRules: "str" = None,
        language: "str" = None,
        text: "Narrative" = None,
        contained: list["Resource"] = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        url: "str" = None,
        identifier: list["Identifier"] = None,
        version: "str" = None,
        name: "str" = None,
        title: "str" = None,
        status: "str" = None,
        experimental: "bool" = None,
        date: "str" = None,
        publisher: "str" = None,
        contact: list["ContactDetail"] = None,
        description: "str" = None,
        useContext: list["UsageContext"] = None,
        jurisdiction: list["CodeableConcept"] = None,
        immutable: "bool" = None,
        purpose: "str" = None,
        copyright: "str" = None,
        compose: "Compose" = None,
        expansion: "Expansion" = None,
    ):

        self.resourceType = "ValueSet"

        self.id = id
        self.meta = meta
        self.implicitRules = implicitRules
        self.language = language
        self.text = text
        self.contained = contained or []
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.url = url
        self.identifier = identifier or []
        self.version = version
        self.name = name
        self.title = title
        self.status = status
        self.experimental = experimental
        self.date = date
        self.publisher = publisher
        self.contact = contact or []
        self.description = description
        self.useContext = useContext or []
        self.jurisdiction = jurisdiction or []
        self.immutable = immutable
        self.purpose = purpose
        self.copyright = copyright
        self.compose = compose
        self.expansion = expansion

    @classmethod
    def from_dict(cls, data: dict) -> "ValueSet":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "ValueSet":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()



"""
Generated class for Endpoint. 
Time: 2024-06-14 18:37:55
"""

from fhirmodels.R5.BackboneElement import *
from fhirmodels.R5.CodeableConcept import *
from fhirmodels.R5.ContactPoint import *
from fhirmodels.R5.DomainResource import *
from fhirmodels.R5.Extension import *
from fhirmodels.R5.Identifier import *
from fhirmodels.R5.Meta import *
from fhirmodels.R5.Narrative import *
from fhirmodels.R5.Period import *
from fhirmodels.R5.Reference import *
from fhirmodels.R5.Resource import *


class Payload(FhirBaseModel):
    """The set of payloads that are provided/available at this endpoint.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param CodeableConcept type: The type of content that may be used at this endpoint (e.g. XDS Discharge summaries)
    :param str mimeType: Mimetype to send. If not specified, the content could be anything (including no payload, if the connectionType defined this)
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "type": {"class_name": "CodeableConcept", "is_contained": False},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        type: list["CodeableConcept"] = None,
        mimeType: list["str"] = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.type = type or []
        self.mimeType = mimeType or []

    @classmethod
    def from_dict(cls, data: dict) -> "Endpoint":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "Endpoint":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class Endpoint(DomainResource):
    """The technical details of an endpoint that can be used for electronic services, such as for web services providing XDS.b, a REST endpoint for another FHIR server, or a s/Mime email address. This may include any security context information.
    :param str id: Logical id of this artifact
    :param Meta meta: Metadata about the resource
    :param str implicitRules: A set of rules under which this content was created
    :param str language: Language of the resource content
    :param Narrative text: Text summary of the resource, for human interpretation
    :param Resource contained: Contained, inline Resources
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored
    :param Identifier identifier: Identifies this endpoint across multiple systems
    :param str status: active | suspended | error | off | entered-in-error | test
    :param CodeableConcept connectionType: Protocol/Profile/Standard to be used with this endpoint connection
    :param str name: A name that this endpoint can be identified by
    :param str description: Additional details about the endpoint that could be displayed as further information to identify the description beyond its name
    :param CodeableConcept environmentType: The type of environment(s) exposed at this endpoint
    :param Reference managingOrganization: Organization that manages this endpoint (might not be the organization that exposes the endpoint)
    :param ContactPoint contact: Contact details for source (e.g. troubleshooting)
    :param Period period: Interval the endpoint is expected to be operational
    :param Payload payload: Set of payloads that are provided by this endpoint
    :param str address: The technical base address for connecting to this endpoint
    :param str header: Usage depends on the channel type
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "meta": {"class_name": "Meta", "is_contained": False},
        "text": {"class_name": "Narrative", "is_contained": False},
        "contained": {"class_name": "Resource", "is_contained": False},
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "identifier": {"class_name": "Identifier", "is_contained": False},
        "connectionType": {"class_name": "CodeableConcept", "is_contained": False},
        "environmentType": {"class_name": "CodeableConcept", "is_contained": False},
        "managingOrganization": {"class_name": "Reference", "is_contained": False},
        "contact": {"class_name": "ContactPoint", "is_contained": False},
        "period": {"class_name": "Period", "is_contained": False},
        "payload": {"class_name": "Payload", "is_contained": True},
    }

    def __init__(
        self,
        id: "str" = None,
        meta: "Meta" = None,
        implicitRules: "str" = None,
        language: "str" = None,
        text: "Narrative" = None,
        contained: list["Resource"] = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        identifier: list["Identifier"] = None,
        status: "str" = None,
        connectionType: list["CodeableConcept"] = None,
        name: "str" = None,
        description: "str" = None,
        environmentType: list["CodeableConcept"] = None,
        managingOrganization: "Reference" = None,
        contact: list["ContactPoint"] = None,
        period: "Period" = None,
        payload: list["Payload"] = None,
        address: "str" = None,
        header: list["str"] = None,
    ):

        self.resourceType = "Endpoint"

        self.id = id
        self.meta = meta
        self.implicitRules = implicitRules
        self.language = language
        self.text = text
        self.contained = contained or []
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.identifier = identifier or []
        self.status = status
        self.connectionType = connectionType or []
        self.name = name
        self.description = description
        self.environmentType = environmentType or []
        self.managingOrganization = managingOrganization
        self.contact = contact or []
        self.period = period
        self.payload = payload or []
        self.address = address
        self.header = header or []

    @classmethod
    def from_dict(cls, data: dict) -> "Endpoint":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "Endpoint":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()



"""
Generated class for ConceptMap. 
Time: 2024-06-14 18:37:49
"""

from fhirmodels.R4B.BackboneElement import *
from fhirmodels.R4B.CodeableConcept import *
from fhirmodels.R4B.ContactDetail import *
from fhirmodels.R4B.DomainResource import *
from fhirmodels.R4B.Extension import *
from fhirmodels.R4B.Identifier import *
from fhirmodels.R4B.Meta import *
from fhirmodels.R4B.Narrative import *
from fhirmodels.R4B.Resource import *
from fhirmodels.R4B.UsageContext import *


class DependsOn(FhirBaseModel):
    """A set of additional dependencies for this mapping to hold. This mapping is only applicable if the specified element can be resolved, and it has the specified value.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param str property: Reference to property mapping depends on
    :param str system: Code System (if necessary)
    :param str value: Value of the referenced element
    :param str display: Display for the code (if value is a code)
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        property: "str" = None,
        system: "str" = None,
        value: "str" = None,
        display: "str" = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.property = property
        self.system = system
        self.value = value
        self.display = display

    @classmethod
    def from_dict(cls, data: dict) -> "ConceptMap":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "ConceptMap":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class Target(FhirBaseModel):
    """A concept from the target value set that this concept maps to.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param str code: Code that identifies the target element
    :param str display: Display for the code
    :param str equivalence: relatedto | equivalent | equal | wider | subsumes | narrower | specializes | inexact | unmatched | disjoint
    :param str comment: Description of status/issues in mapping
    :param DependsOn dependsOn: Other elements required for this mapping (from context)
    :param Product product: Other concepts that this mapping also produces
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "dependsOn": {"class_name": "DependsOn", "is_contained": True},
        "product": {"class_name": "Product", "is_contained": True},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        code: "str" = None,
        display: "str" = None,
        equivalence: "str" = None,
        comment: "str" = None,
        dependsOn: list["DependsOn"] = None,
        product: list["Product"] = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.code = code
        self.display = display
        self.equivalence = equivalence
        self.comment = comment
        self.dependsOn = dependsOn or []
        self.product = product or []

    @classmethod
    def from_dict(cls, data: dict) -> "ConceptMap":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "ConceptMap":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class Element(FhirBaseModel):
    """Mappings for an individual concept in the source to one or more concepts in the target.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param str code: Identifies element being mapped
    :param str display: Display for the code
    :param Target target: Concept in target system for element
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "target": {"class_name": "Target", "is_contained": True},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        code: "str" = None,
        display: "str" = None,
        target: list["Target"] = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.code = code
        self.display = display
        self.target = target or []

    @classmethod
    def from_dict(cls, data: dict) -> "ConceptMap":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "ConceptMap":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class Unmapped(FhirBaseModel):
    """What to do when there is no mapping for the source concept. "Unmapped" does not include codes that are unmatched, and the unmapped element is ignored in a code is specified to have equivalence = unmatched.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param str mode: provided | fixed | other-map
    :param str code: Fixed code when mode = fixed
    :param str display: Display for the code
    :param str url: canonical reference to an additional ConceptMap to use for mapping if the source concept is unmapped
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        mode: "str" = None,
        code: "str" = None,
        display: "str" = None,
        url: "str" = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.mode = mode
        self.code = code
        self.display = display
        self.url = url

    @classmethod
    def from_dict(cls, data: dict) -> "ConceptMap":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "ConceptMap":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class Group(FhirBaseModel):
    """A group of mappings that all have the same source and target system.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param str source: Source system where concepts to be mapped are defined
    :param str sourceVersion: Specific version of the  code system
    :param str target: Target system that the concepts are to be mapped to
    :param str targetVersion: Specific version of the  code system
    :param Element element: Mappings for a concept from the source set
    :param Unmapped unmapped: What to do when there is no mapping for the source concept
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "element": {"class_name": "Element", "is_contained": True},
        "unmapped": {"class_name": "Unmapped", "is_contained": True},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        source: "str" = None,
        sourceVersion: "str" = None,
        target: "str" = None,
        targetVersion: "str" = None,
        element: list["Element"] = None,
        unmapped: "Unmapped" = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.source = source
        self.sourceVersion = sourceVersion
        self.target = target
        self.targetVersion = targetVersion
        self.element = element or []
        self.unmapped = unmapped

    @classmethod
    def from_dict(cls, data: dict) -> "ConceptMap":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "ConceptMap":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class ConceptMap(DomainResource):
    """A statement of relationships from one set of concepts to one or more other concepts - either concepts in code systems, or data element/data element concepts, or classes in class models.
    :param str id: Logical id of this artifact
    :param Meta meta: Metadata about the resource
    :param str implicitRules: A set of rules under which this content was created
    :param str language: Language of the resource content
    :param Narrative text: Text summary of the resource, for human interpretation
    :param Resource contained: Contained, inline Resources
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored
    :param str url: Canonical identifier for this concept map, represented as a URI (globally unique)
    :param Identifier identifier: Additional identifier for the concept map
    :param str version: Business version of the concept map
    :param str name: Name for this concept map (computer friendly)
    :param str title: Name for this concept map (human friendly)
    :param str status: draft | active | retired | unknown
    :param bool experimental: For testing purposes, not real usage
    :param str date: Date last changed
    :param str publisher: Name of the publisher (organization or individual)
    :param ContactDetail contact: Contact details for the publisher
    :param str description: Natural language description of the concept map
    :param UsageContext useContext: The context that the content is intended to support
    :param CodeableConcept jurisdiction: Intended jurisdiction for concept map (if applicable)
    :param str purpose: Why this concept map is defined
    :param str copyright: Use and/or publishing restrictions
    :param str sourceUri: The source value set that contains the concepts that are being mapped
    :param str sourceCanonical: The source value set that contains the concepts that are being mapped
    :param str targetUri: The target value set which provides context for the mappings
    :param str targetCanonical: The target value set which provides context for the mappings
    :param Group group: Same source and target systems
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "meta": {"class_name": "Meta", "is_contained": False},
        "text": {"class_name": "Narrative", "is_contained": False},
        "contained": {"class_name": "Resource", "is_contained": False},
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "identifier": {"class_name": "Identifier", "is_contained": False},
        "contact": {"class_name": "ContactDetail", "is_contained": False},
        "useContext": {"class_name": "UsageContext", "is_contained": False},
        "jurisdiction": {"class_name": "CodeableConcept", "is_contained": False},
        "group": {"class_name": "Group", "is_contained": True},
    }

    def __init__(
        self,
        id: "str" = None,
        meta: "Meta" = None,
        implicitRules: "str" = None,
        language: "str" = None,
        text: "Narrative" = None,
        contained: list["Resource"] = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        url: "str" = None,
        identifier: "Identifier" = None,
        version: "str" = None,
        name: "str" = None,
        title: "str" = None,
        status: "str" = None,
        experimental: "bool" = None,
        date: "str" = None,
        publisher: "str" = None,
        contact: list["ContactDetail"] = None,
        description: "str" = None,
        useContext: list["UsageContext"] = None,
        jurisdiction: list["CodeableConcept"] = None,
        purpose: "str" = None,
        copyright: "str" = None,
        sourceUri: "str" = None,
        sourceCanonical: "str" = None,
        targetUri: "str" = None,
        targetCanonical: "str" = None,
        group: list["Group"] = None,
    ):

        self.resourceType = "ConceptMap"

        self.id = id
        self.meta = meta
        self.implicitRules = implicitRules
        self.language = language
        self.text = text
        self.contained = contained or []
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.url = url
        self.identifier = identifier
        self.version = version
        self.name = name
        self.title = title
        self.status = status
        self.experimental = experimental
        self.date = date
        self.publisher = publisher
        self.contact = contact or []
        self.description = description
        self.useContext = useContext or []
        self.jurisdiction = jurisdiction or []
        self.purpose = purpose
        self.copyright = copyright
        self.sourceUri = sourceUri
        self.sourceCanonical = sourceCanonical
        self.targetUri = targetUri
        self.targetCanonical = targetCanonical
        self.group = group or []

    @classmethod
    def from_dict(cls, data: dict) -> "ConceptMap":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "ConceptMap":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()



import io
import json
import logging
import os
import tarfile
import typing as t
from dataclasses import dataclass

import requests

from fhirmodels import constants as _c
from fhirmodels import utils

logger = logging.getLogger(__name__)


@dataclass(frozen=True)
class IgDependency:
    """The URL and version of a package dependency."""

    url: str
    version: str


@dataclass(frozen=True)
class PackageMaintainer:
    """The name and email of a package maintainer."""

    name: str
    email: str


@dataclass(frozen=True)
class PackageInfo:
    """Metadata parsed from a package's package.json file.
    See documentation for the package.json file at:
    https://confluence.hl7.org/pages/viewpage.action?pageId=35718629#NPMPackageSpecification-Packagemanifest
    """

    name: str
    version: str
    description: str | None
    canonical: str | None
    title: str | None
    url: str | None
    fhirVersions: list[str] | None
    dependencies: t.Collection["IgDependency"] | None
    keywords: tuple[str] | None
    author: str | None
    maintainers: list["PackageMaintainer"] | None
    jurisdiction: str | None
    license: str | None


class FhirPackageLoader:
    """Loads a FHIR package from a `.tar.gz` or `.tgz` file following NPM conventions."""

    def __init__(self):
        pass

    def load_from_version(
        self, fhir_version: t.Literal["R4", "R4B", "R5"]
    ) -> "FhirPackage":
        """Loads a FHIR package for a specified FHIR version."""
        version_urls = _c.FHIR_VERSION_PACKAGE_URLS
        url = version_urls[fhir_version]
        return self.load_from_npm(url, name=fhir_version)

    def load_from_path(self, path: str, name: str = None) -> "FhirPackage":
        """Loads a FHIR package from a local path."""
        package = FhirPackage()
        # check if valid directory
        if not os.path.isdir(path):
            raise ValueError(f"Path is not a directory: {path}")
        for file in utils.walk_directory(path):
            if not utils.is_json_file(file):
                continue
            filename = os.path.basename(file)
            content = utils.read_json_file(file)
            resource_type = content.get("resourceType")
            package.add_file(
                data=content, filename=filename, resource_type=resource_type
            )
        return package

    def load_from_npm(self, url: str, name: str = None):
        """Loads a FHIR npm package from an URL."""
        with requests.get(url, stream=True) as res:
            res.raise_for_status()
            package_bytes = res.content
            package_buffer = io.BytesIO(package_bytes)
        return FhirPackage.from_npm_file(npm_file=package_buffer, name=name)

    def load_from_simplifier(self, name: str, version: str):
        """Loads a FHIR npm package from simplifier.net."""
        url = f"https://packages.simplifier.net/{name}/{version}"
        return self.load_from_npm(url, name=name)


class FhirPackage:
    """Represents a FHIR package. In case a FHIR base version is loaded,
    the name should be equivalent to the FHIR version (e.g. "R4")."""

    def __init__(
        self,
        package_info: PackageInfo | None = None,
        code_systems: list[dict] | None = None,
        search_parameters: list[dict] | None = None,
        structure_definitions: list[dict] | None = None,
        value_sets: list[dict] | None = None,
        capability_statements: list[dict] | None = None,
        concept_maps: list[dict] | None = None,
        name: str | None = None,
    ):
        self.package_info = package_info
        self.code_systems = code_systems or []
        self.search_parameters = search_parameters or []
        self.structure_definitions = structure_definitions or []
        self.value_sets = value_sets or []
        self.capability_statements = capability_statements or []
        self.concept_maps = concept_maps or []
        self.name = name

    @classmethod
    def from_npm_file(cls, npm_file: t.BinaryIO, name: str = None):
        """Loads a FHIR package from a `.tar.gz` or `.tgz` file following NPM conventions."""
        package_info = None
        code_systems = []
        search_parameters = []
        structure_definitions = []
        value_sets = []
        capability_statements = []
        concept_maps = []
        for filename, content in _read_fhir_package_npm(npm_file):
            content_json: dict = json.loads(content)
            resource_type = content_json.get("resourceType")
            if os.path.basename(filename) == "package.json":
                package_info = _parse_package_info(content_json)
            if resource_type == _c.CODESYSTEM:
                code_systems.append(content_json)
            elif resource_type == _c.SEARCHPARAMETER:
                search_parameters.append(content_json)
            elif resource_type == _c.STRUCTUREDEFINITION:
                structure_definitions.append(content_json)
            elif resource_type == _c.VALUESET:
                value_sets.append(content_json)
            elif resource_type == _c.CAPABILITYSTATEMENT:
                capability_statements.append(content_json)
            elif resource_type == _c.CONCEPTMAP:
                concept_maps.append(content_json)
            else:
                # skipping file
                continue

        return cls(
            package_info=package_info,
            code_systems=code_systems,
            search_parameters=search_parameters,
            structure_definitions=structure_definitions,
            value_sets=value_sets,
            capability_statements=capability_statements,
            concept_maps=concept_maps,
            name=name,
        )

    def get_codesystem_by_url(self, url: str):
        for cs in self.code_systems:
            if cs["url"] == url:
                return cs
        return None

    @property
    def resource_structure_definitions(self):
        return self._get_structure_definitions_by_kind("resource")

    @property
    def base_resource_structure_definitions(self):
        resources = self._get_structure_definitions_by_kind("resource")
        base_resources = [
            r
            for r in resources
            if r.get("baseDefinition") == _c.DOMAIN_RESOURCE_STRUC_DEF_URL
        ]
        return base_resources

    @property
    def complex_type_structure_definitions(self):
        return self._get_structure_definitions_by_kind("complex-type")

    @property
    def primitive_type_structure_definitions(self):
        return self._get_structure_definitions_by_kind("primitive-type")

    @property
    def logical_structure_definitions(self):
        return self._get_structure_definitions_by_kind("logical")

    @property
    def base_capability_statement(self) -> dict | None:
        """Packages might contain multiple capability statements.
        This method returns the base capability statement identified by the id.
        """
        for cs in self.capability_statements:
            if cs["id"] == "base":
                return cs
        return None

    def add_file(
        self,
        data: dict,
        filename: str,
        resource_type: str | None = None,
    ):
        if filename == "package.json":
            package_info = _parse_package_info(data)
            self.package_info = package_info
        elif resource_type == _c.CODESYSTEM:
            self.add_code_system(data)
        elif resource_type == _c.SEARCHPARAMETER:
            self.add_search_parameter(data)
        elif resource_type == _c.STRUCTUREDEFINITION:
            self.add_structure_definition(data)
        elif resource_type == _c.VALUESET:
            self.add_value_set(data)
        elif resource_type == _c.CAPABILITYSTATEMENT:
            self.add_capability_statement(data)
        elif resource_type == _c.CONCEPTMAP:
            self.add_concept_map(data)
        else:
            # skipping file
            return

    def add_structure_definition(self, structure_definition: dict):
        self.structure_definitions.append(structure_definition)

    def add_code_system(self, code_system: dict):
        self.code_systems.append(code_system)

    def add_search_parameter(self, search_parameter: dict):
        self.search_parameters.append(search_parameter)

    def add_value_set(self, value_set: dict):
        self.value_sets.append(value_set)

    def add_capability_statement(self, capability_statement: dict):
        self.capability_statements.append(capability_statement)

    def add_concept_map(self, concept_map: dict):
        self.concept_maps.append(concept_map)

    def _get_structure_definitions_by_kind(
        self, kind: t.Literal["resource", "complex-type", "primitive-type", "logical"]
    ):
        return [sd for sd in self.structure_definitions if sd["kind"] == kind]


def _read_fhir_package_npm(npm_file: t.BinaryIO) -> t.Iterator[t.Tuple[str, str]]:
    """Yields the file entries for JSON resources in `npm_file` and their contents."""
    with tarfile.open(fileobj=npm_file, mode="r:gz") as f:
        for member in f.getmembers():
            if member.name.endswith(".json"):
                content = f.extractfile(member)
                if content is not None:
                    # using utf-8-sig to remove dealing with BOM, https://stackoverflow.com/questions/57152985/what-is-the-difference-between-utf-8-and-utf-8-sig
                    yield member.name, content.read().decode("utf-8-sig")
            else:
                # logger.info("Skipping  entry: %s.", member.name)
                continue


def _parse_package_info(json_obj: dict[str, t.Any]) -> "PackageInfo":
    """Creates an PackageInfo object given the contents of a package.json file."""
    return PackageInfo(
        name=json_obj["name"],
        version=json_obj["version"],
        canonical=json_obj.get("canonical"),
        title=json_obj.get("title"),
        description=json_obj.get("description"),
        dependencies=tuple(
            IgDependency(url=url, version=version)
            for url, version in json_obj.get("dependencies", {}).items()
        ),
        author=json_obj.get("author"),
        fhirVersions=json_obj.get("fhirVersions"),
        jurisdiction=json_obj.get("jurisdiction"),
        keywords=json_obj.get("keywords"),
        license=json_obj.get("license"),
        maintainers=[
            PackageMaintainer(
                name=maintainer.get("name", ""), email=maintainer.get("email", "")
            )
            for maintainer in json_obj.get("maintainers", [])
        ],
        url=json_obj.get("url"),
    )



"""
Generated class for Appointment. 
Time: 2024-06-14 18:37:49
"""

from fhirmodels.R4B.BackboneElement import *
from fhirmodels.R4B.CodeableConcept import *
from fhirmodels.R4B.DomainResource import *
from fhirmodels.R4B.Extension import *
from fhirmodels.R4B.Identifier import *
from fhirmodels.R4B.Meta import *
from fhirmodels.R4B.Narrative import *
from fhirmodels.R4B.Period import *
from fhirmodels.R4B.Reference import *
from fhirmodels.R4B.Resource import *


class Participant(FhirBaseModel):
    """List of participants involved in the appointment.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param CodeableConcept type: Role of participant in the appointment
    :param Reference actor: Person, Location/HealthcareService or Device
    :param str required: required | optional | information-only
    :param str status: accepted | declined | tentative | needs-action
    :param Period period: Participation period of the actor
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "type": {"class_name": "CodeableConcept", "is_contained": False},
        "actor": {"class_name": "Reference", "is_contained": False},
        "period": {"class_name": "Period", "is_contained": False},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        type: list["CodeableConcept"] = None,
        actor: "Reference" = None,
        required: "str" = None,
        status: "str" = None,
        period: "Period" = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.type = type or []
        self.actor = actor
        self.required = required
        self.status = status
        self.period = period

    @classmethod
    def from_dict(cls, data: dict) -> "Appointment":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "Appointment":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class Appointment(DomainResource):
    """A booking of a healthcare event among patient(s), practitioner(s), related person(s) and/or device(s) for a specific date/time. This may result in one or more Encounter(s).
    :param str id: Logical id of this artifact
    :param Meta meta: Metadata about the resource
    :param str implicitRules: A set of rules under which this content was created
    :param str language: Language of the resource content
    :param Narrative text: Text summary of the resource, for human interpretation
    :param Resource contained: Contained, inline Resources
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored
    :param Identifier identifier: External Ids for this item
    :param str status: proposed | pending | booked | arrived | fulfilled | cancelled | noshow | entered-in-error | checked-in | waitlist
    :param CodeableConcept cancelationReason: The coded reason for the appointment being cancelled
    :param CodeableConcept serviceCategory: A broad categorization of the service that is to be performed during this appointment
    :param CodeableConcept serviceType: The specific service that is to be performed during this appointment
    :param CodeableConcept specialty: The specialty of a practitioner that would be required to perform the service requested in this appointment
    :param CodeableConcept appointmentType: The style of appointment or patient that has been booked in the slot (not service type)
    :param CodeableConcept reasonCode: Coded reason this appointment is scheduled
    :param Reference reasonReference: Reason the appointment is to take place (resource)
    :param int priority: Used to make informed decisions if needing to re-prioritize
    :param str description: Shown on a subject line in a meeting request, or appointment list
    :param Reference supportingInformation: Additional information to support the appointment
    :param str start: When appointment is to take place
    :param str end: When appointment is to conclude
    :param int minutesDuration: Can be less than start/end (e.g. estimate)
    :param Reference slot: The slots that this appointment is filling
    :param str created: The date that this appointment was initially created
    :param str comment: Additional comments
    :param str patientInstruction: Detailed information and instructions for the patient
    :param Reference basedOn: The service request this appointment is allocated to assess
    :param Participant participant: Participants involved in appointment
    :param Period requestedPeriod: Potential date/time interval(s) requested to allocate the appointment within
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "meta": {"class_name": "Meta", "is_contained": False},
        "text": {"class_name": "Narrative", "is_contained": False},
        "contained": {"class_name": "Resource", "is_contained": False},
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "identifier": {"class_name": "Identifier", "is_contained": False},
        "cancelationReason": {"class_name": "CodeableConcept", "is_contained": False},
        "serviceCategory": {"class_name": "CodeableConcept", "is_contained": False},
        "serviceType": {"class_name": "CodeableConcept", "is_contained": False},
        "specialty": {"class_name": "CodeableConcept", "is_contained": False},
        "appointmentType": {"class_name": "CodeableConcept", "is_contained": False},
        "reasonCode": {"class_name": "CodeableConcept", "is_contained": False},
        "reasonReference": {"class_name": "Reference", "is_contained": False},
        "supportingInformation": {"class_name": "Reference", "is_contained": False},
        "slot": {"class_name": "Reference", "is_contained": False},
        "basedOn": {"class_name": "Reference", "is_contained": False},
        "participant": {"class_name": "Participant", "is_contained": True},
        "requestedPeriod": {"class_name": "Period", "is_contained": False},
    }

    def __init__(
        self,
        id: "str" = None,
        meta: "Meta" = None,
        implicitRules: "str" = None,
        language: "str" = None,
        text: "Narrative" = None,
        contained: list["Resource"] = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        identifier: list["Identifier"] = None,
        status: "str" = None,
        cancelationReason: "CodeableConcept" = None,
        serviceCategory: list["CodeableConcept"] = None,
        serviceType: list["CodeableConcept"] = None,
        specialty: list["CodeableConcept"] = None,
        appointmentType: "CodeableConcept" = None,
        reasonCode: list["CodeableConcept"] = None,
        reasonReference: list["Reference"] = None,
        priority: "int" = None,
        description: "str" = None,
        supportingInformation: list["Reference"] = None,
        start: "str" = None,
        end: "str" = None,
        minutesDuration: "int" = None,
        slot: list["Reference"] = None,
        created: "str" = None,
        comment: "str" = None,
        patientInstruction: "str" = None,
        basedOn: list["Reference"] = None,
        participant: list["Participant"] = None,
        requestedPeriod: list["Period"] = None,
    ):

        self.resourceType = "Appointment"

        self.id = id
        self.meta = meta
        self.implicitRules = implicitRules
        self.language = language
        self.text = text
        self.contained = contained or []
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.identifier = identifier or []
        self.status = status
        self.cancelationReason = cancelationReason
        self.serviceCategory = serviceCategory or []
        self.serviceType = serviceType or []
        self.specialty = specialty or []
        self.appointmentType = appointmentType
        self.reasonCode = reasonCode or []
        self.reasonReference = reasonReference or []
        self.priority = priority
        self.description = description
        self.supportingInformation = supportingInformation or []
        self.start = start
        self.end = end
        self.minutesDuration = minutesDuration
        self.slot = slot or []
        self.created = created
        self.comment = comment
        self.patientInstruction = patientInstruction
        self.basedOn = basedOn or []
        self.participant = participant or []
        self.requestedPeriod = requestedPeriod or []

    @classmethod
    def from_dict(cls, data: dict) -> "Appointment":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "Appointment":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()



"""
Generated class for Shareable. 
Time: 2024-06-14 18:37:55
"""

from fhirmodels.generator import FhirBaseModel
from fhirmodels.R5.CodeableConcept import *
from fhirmodels.R5.Coding import *


class Shareable(FhirBaseModel):
    """Logical Model: A pattern to be followed by [canonical resources](canonicalresource.html) that meet minimal requirements for sharing via registries or similar mechanisms.
    :param str url: Canonical identifier for this {{title}}, represented as a URI (globally unique)
    :param str version: Business version of the {{title}}
    :param str versionAlgorithmString: How to compare versions
    :param Coding versionAlgorithmCoding: How to compare versions
    :param str name: Name for this {{title}} (computer friendly)
    :param str title: Name for this {{title}} (human-friendly)
    :param str status: draft | active | retired | unknown
    :param bool experimental: For testing purposes, not real usage
    :param str publisher: Name of the publisher (or steward) (organization or individual)
    :param str description: Natural language description of the {{title}}
    :param CodeableConcept knowledgeRepresentationLevel: narrative | semi-structured | structured | executable
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "versionAlgorithmCoding": {"class_name": "Coding", "is_contained": False},
        "knowledgeRepresentationLevel": {
            "class_name": "CodeableConcept",
            "is_contained": False,
        },
    }

    def __init__(
        self,
        url: "str" = None,
        version: "str" = None,
        versionAlgorithmString: "str" = None,
        versionAlgorithmCoding: "Coding" = None,
        name: "str" = None,
        title: "str" = None,
        status: "str" = None,
        experimental: "bool" = None,
        publisher: "str" = None,
        description: "str" = None,
        knowledgeRepresentationLevel: list["CodeableConcept"] = None,
    ):

        self.url = url
        self.version = version
        self.versionAlgorithmString = versionAlgorithmString
        self.versionAlgorithmCoding = versionAlgorithmCoding
        self.name = name
        self.title = title
        self.status = status
        self.experimental = experimental
        self.publisher = publisher
        self.description = description
        self.knowledgeRepresentationLevel = knowledgeRepresentationLevel or []

    @classmethod
    def from_dict(cls, data: dict) -> "Shareable":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "Shareable":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()



"""
Generated class for Questionnaire. 
Time: 2024-06-14 18:37:49
"""

from fhirmodels.R4B.Attachment import *
from fhirmodels.R4B.BackboneElement import *
from fhirmodels.R4B.CodeableConcept import *
from fhirmodels.R4B.Coding import *
from fhirmodels.R4B.ContactDetail import *
from fhirmodels.R4B.DomainResource import *
from fhirmodels.R4B.Extension import *
from fhirmodels.R4B.Identifier import *
from fhirmodels.R4B.Meta import *
from fhirmodels.R4B.Narrative import *
from fhirmodels.R4B.Period import *
from fhirmodels.R4B.Quantity import *
from fhirmodels.R4B.Reference import *
from fhirmodels.R4B.Resource import *
from fhirmodels.R4B.UsageContext import *


class EnableWhen(FhirBaseModel):
    """A constraint indicating that this item should only be enabled (displayed/allow answers to be captured) when the specified condition is true.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param str question: Question that determines whether item is enabled
    :param str operator: exists | = | != | > | < | >= | <=
    :param bool answerBoolean: Value for question comparison based on operator
    :param float answerDecimal: Value for question comparison based on operator
    :param int answerInteger: Value for question comparison based on operator
    :param str answerDate: Value for question comparison based on operator
    :param str answerDateTime: Value for question comparison based on operator
    :param str answerTime: Value for question comparison based on operator
    :param str answerString: Value for question comparison based on operator
    :param Coding answerCoding: Value for question comparison based on operator
    :param Quantity answerQuantity: Value for question comparison based on operator
    :param Reference answerReference: Value for question comparison based on operator
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "answerCoding": {"class_name": "Coding", "is_contained": False},
        "answerQuantity": {"class_name": "Quantity", "is_contained": False},
        "answerReference": {"class_name": "Reference", "is_contained": False},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        question: "str" = None,
        operator: "str" = None,
        answerBoolean: "bool" = None,
        answerDecimal: "float" = None,
        answerInteger: "int" = None,
        answerDate: "str" = None,
        answerDateTime: "str" = None,
        answerTime: "str" = None,
        answerString: "str" = None,
        answerCoding: "Coding" = None,
        answerQuantity: "Quantity" = None,
        answerReference: "Reference" = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.question = question
        self.operator = operator
        self.answerBoolean = answerBoolean
        self.answerDecimal = answerDecimal
        self.answerInteger = answerInteger
        self.answerDate = answerDate
        self.answerDateTime = answerDateTime
        self.answerTime = answerTime
        self.answerString = answerString
        self.answerCoding = answerCoding
        self.answerQuantity = answerQuantity
        self.answerReference = answerReference

    @classmethod
    def from_dict(cls, data: dict) -> "Questionnaire":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "Questionnaire":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class AnswerOption(FhirBaseModel):
    """One of the permitted answers for a "choice" or "open-choice" question.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param int valueInteger: Answer value
    :param str valueDate: Answer value
    :param str valueTime: Answer value
    :param str valueString: Answer value
    :param Coding valueCoding: Answer value
    :param Reference valueReference: Answer value
    :param bool initialSelected: Whether option is selected by default
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "valueCoding": {"class_name": "Coding", "is_contained": False},
        "valueReference": {"class_name": "Reference", "is_contained": False},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        valueInteger: "int" = None,
        valueDate: "str" = None,
        valueTime: "str" = None,
        valueString: "str" = None,
        valueCoding: "Coding" = None,
        valueReference: "Reference" = None,
        initialSelected: "bool" = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.valueInteger = valueInteger
        self.valueDate = valueDate
        self.valueTime = valueTime
        self.valueString = valueString
        self.valueCoding = valueCoding
        self.valueReference = valueReference
        self.initialSelected = initialSelected

    @classmethod
    def from_dict(cls, data: dict) -> "Questionnaire":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "Questionnaire":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class Initial(FhirBaseModel):
    """One or more values that should be pre-populated in the answer when initially rendering the questionnaire for user input.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param bool valueBoolean: Actual value for initializing the question
    :param float valueDecimal: Actual value for initializing the question
    :param int valueInteger: Actual value for initializing the question
    :param str valueDate: Actual value for initializing the question
    :param str valueDateTime: Actual value for initializing the question
    :param str valueTime: Actual value for initializing the question
    :param str valueString: Actual value for initializing the question
    :param str valueUri: Actual value for initializing the question
    :param Attachment valueAttachment: Actual value for initializing the question
    :param Coding valueCoding: Actual value for initializing the question
    :param Quantity valueQuantity: Actual value for initializing the question
    :param Reference valueReference: Actual value for initializing the question
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "valueAttachment": {"class_name": "Attachment", "is_contained": False},
        "valueCoding": {"class_name": "Coding", "is_contained": False},
        "valueQuantity": {"class_name": "Quantity", "is_contained": False},
        "valueReference": {"class_name": "Reference", "is_contained": False},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        valueBoolean: "bool" = None,
        valueDecimal: "float" = None,
        valueInteger: "int" = None,
        valueDate: "str" = None,
        valueDateTime: "str" = None,
        valueTime: "str" = None,
        valueString: "str" = None,
        valueUri: "str" = None,
        valueAttachment: "Attachment" = None,
        valueCoding: "Coding" = None,
        valueQuantity: "Quantity" = None,
        valueReference: "Reference" = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.valueBoolean = valueBoolean
        self.valueDecimal = valueDecimal
        self.valueInteger = valueInteger
        self.valueDate = valueDate
        self.valueDateTime = valueDateTime
        self.valueTime = valueTime
        self.valueString = valueString
        self.valueUri = valueUri
        self.valueAttachment = valueAttachment
        self.valueCoding = valueCoding
        self.valueQuantity = valueQuantity
        self.valueReference = valueReference

    @classmethod
    def from_dict(cls, data: dict) -> "Questionnaire":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "Questionnaire":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class Item(FhirBaseModel):
    """A particular question, question grouping or display text that is part of the questionnaire.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param str linkId: Unique id for item in questionnaire
    :param str definition: ElementDefinition - details for the item
    :param Coding code: Corresponding concept for this item in a terminology
    :param str prefix: E.g. "1(a)", "2.5.3"
    :param str text: Primary text for the item
    :param str type: group | display | boolean | decimal | integer | date | dateTime +
    :param EnableWhen enableWhen: Only allow data when
    :param str enableBehavior: all | any
    :param bool required: Whether the item must be included in data results
    :param bool repeats: Whether the item may repeat
    :param bool readOnly: Don't allow human editing
    :param int maxLength: No more than this many characters
    :param str answerValueSet: Valueset containing permitted answers
    :param AnswerOption answerOption: Permitted answer
    :param Initial initial: Initial value(s) when item is first rendered
    :param Item item: Nested questionnaire items
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "code": {"class_name": "Coding", "is_contained": False},
        "enableWhen": {"class_name": "EnableWhen", "is_contained": True},
        "answerOption": {"class_name": "AnswerOption", "is_contained": True},
        "initial": {"class_name": "Initial", "is_contained": True},
        "item": {"class_name": "Item", "is_contained": True},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        linkId: "str" = None,
        definition: "str" = None,
        code: list["Coding"] = None,
        prefix: "str" = None,
        text: "str" = None,
        type: "str" = None,
        enableWhen: list["EnableWhen"] = None,
        enableBehavior: "str" = None,
        required: "bool" = None,
        repeats: "bool" = None,
        readOnly: "bool" = None,
        maxLength: "int" = None,
        answerValueSet: "str" = None,
        answerOption: list["AnswerOption"] = None,
        initial: list["Initial"] = None,
        item: list["Item"] = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.linkId = linkId
        self.definition = definition
        self.code = code or []
        self.prefix = prefix
        self.text = text
        self.type = type
        self.enableWhen = enableWhen or []
        self.enableBehavior = enableBehavior
        self.required = required
        self.repeats = repeats
        self.readOnly = readOnly
        self.maxLength = maxLength
        self.answerValueSet = answerValueSet
        self.answerOption = answerOption or []
        self.initial = initial or []
        self.item = item or []

    @classmethod
    def from_dict(cls, data: dict) -> "Questionnaire":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "Questionnaire":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class Questionnaire(DomainResource):
    """A structured set of questions intended to guide the collection of answers from end-users. Questionnaires provide detailed control over order, presentation, phraseology and grouping to allow coherent, consistent data collection.
    :param str id: Logical id of this artifact
    :param Meta meta: Metadata about the resource
    :param str implicitRules: A set of rules under which this content was created
    :param str language: Language of the resource content
    :param Narrative text: Text summary of the resource, for human interpretation
    :param Resource contained: Contained, inline Resources
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored
    :param str url: Canonical identifier for this questionnaire, represented as a URI (globally unique)
    :param Identifier identifier: Additional identifier for the questionnaire
    :param str version: Business version of the questionnaire
    :param str name: Name for this questionnaire (computer friendly)
    :param str title: Name for this questionnaire (human friendly)
    :param str derivedFrom: Instantiates protocol or definition
    :param str status: draft | active | retired | unknown
    :param bool experimental: For testing purposes, not real usage
    :param str subjectType: Resource that can be subject of QuestionnaireResponse
    :param str date: Date last changed
    :param str publisher: Name of the publisher (organization or individual)
    :param ContactDetail contact: Contact details for the publisher
    :param str description: Natural language description of the questionnaire
    :param UsageContext useContext: The context that the content is intended to support
    :param CodeableConcept jurisdiction: Intended jurisdiction for questionnaire (if applicable)
    :param str purpose: Why this questionnaire is defined
    :param str copyright: Use and/or publishing restrictions
    :param str approvalDate: When the questionnaire was approved by publisher
    :param str lastReviewDate: When the questionnaire was last reviewed
    :param Period effectivePeriod: When the questionnaire is expected to be used
    :param Coding code: Concept that represents the overall questionnaire
    :param Item item: Questions and sections within the Questionnaire
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "meta": {"class_name": "Meta", "is_contained": False},
        "text": {"class_name": "Narrative", "is_contained": False},
        "contained": {"class_name": "Resource", "is_contained": False},
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "identifier": {"class_name": "Identifier", "is_contained": False},
        "contact": {"class_name": "ContactDetail", "is_contained": False},
        "useContext": {"class_name": "UsageContext", "is_contained": False},
        "jurisdiction": {"class_name": "CodeableConcept", "is_contained": False},
        "effectivePeriod": {"class_name": "Period", "is_contained": False},
        "code": {"class_name": "Coding", "is_contained": False},
        "item": {"class_name": "Item", "is_contained": True},
    }

    def __init__(
        self,
        id: "str" = None,
        meta: "Meta" = None,
        implicitRules: "str" = None,
        language: "str" = None,
        text: "Narrative" = None,
        contained: list["Resource"] = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        url: "str" = None,
        identifier: list["Identifier"] = None,
        version: "str" = None,
        name: "str" = None,
        title: "str" = None,
        derivedFrom: list["str"] = None,
        status: "str" = None,
        experimental: "bool" = None,
        subjectType: list["str"] = None,
        date: "str" = None,
        publisher: "str" = None,
        contact: list["ContactDetail"] = None,
        description: "str" = None,
        useContext: list["UsageContext"] = None,
        jurisdiction: list["CodeableConcept"] = None,
        purpose: "str" = None,
        copyright: "str" = None,
        approvalDate: "str" = None,
        lastReviewDate: "str" = None,
        effectivePeriod: "Period" = None,
        code: list["Coding"] = None,
        item: list["Item"] = None,
    ):

        self.resourceType = "Questionnaire"

        self.id = id
        self.meta = meta
        self.implicitRules = implicitRules
        self.language = language
        self.text = text
        self.contained = contained or []
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.url = url
        self.identifier = identifier or []
        self.version = version
        self.name = name
        self.title = title
        self.derivedFrom = derivedFrom or []
        self.status = status
        self.experimental = experimental
        self.subjectType = subjectType or []
        self.date = date
        self.publisher = publisher
        self.contact = contact or []
        self.description = description
        self.useContext = useContext or []
        self.jurisdiction = jurisdiction or []
        self.purpose = purpose
        self.copyright = copyright
        self.approvalDate = approvalDate
        self.lastReviewDate = lastReviewDate
        self.effectivePeriod = effectivePeriod
        self.code = code or []
        self.item = item or []

    @classmethod
    def from_dict(cls, data: dict) -> "Questionnaire":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "Questionnaire":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()



"""
Generated class for CodeSystem. 
Time: 2024-06-14 18:37:49
"""

from fhirmodels.R4B.BackboneElement import *
from fhirmodels.R4B.CodeableConcept import *
from fhirmodels.R4B.Coding import *
from fhirmodels.R4B.ContactDetail import *
from fhirmodels.R4B.DomainResource import *
from fhirmodels.R4B.Extension import *
from fhirmodels.R4B.Identifier import *
from fhirmodels.R4B.Meta import *
from fhirmodels.R4B.Narrative import *
from fhirmodels.R4B.Resource import *
from fhirmodels.R4B.UsageContext import *


class Filter(FhirBaseModel):
    """A filter that can be used in a value set compose statement when selecting concepts using a filter.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param str code: Code that identifies the filter
    :param str description: How or why the filter is used
    :param str operator: = | is-a | descendent-of | is-not-a | regex | in | not-in | generalizes | exists
    :param str value: What to use for the value
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        code: "str" = None,
        description: "str" = None,
        operator: list["str"] = None,
        value: "str" = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.code = code
        self.description = description
        self.operator = operator or []
        self.value = value

    @classmethod
    def from_dict(cls, data: dict) -> "CodeSystem":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "CodeSystem":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class Property(FhirBaseModel):
    """A property defines an additional slot through which additional information can be provided about a concept.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param str code: Identifies the property on the concepts, and when referred to in operations
    :param str uri: Formal identifier for the property
    :param str description: Why the property is defined, and/or what it conveys
    :param str type: code | Coding | string | integer | boolean | dateTime | decimal
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        code: "str" = None,
        uri: "str" = None,
        description: "str" = None,
        type: "str" = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.code = code
        self.uri = uri
        self.description = description
        self.type = type

    @classmethod
    def from_dict(cls, data: dict) -> "CodeSystem":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "CodeSystem":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class Designation(FhirBaseModel):
    """Additional representations for the concept - other languages, aliases, specialized purposes, used for particular purposes, etc.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param str language: Human language of the designation
    :param Coding use: Details how this designation would be used
    :param str value: The text value for this designation
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "use": {"class_name": "Coding", "is_contained": False},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        language: "str" = None,
        use: "Coding" = None,
        value: "str" = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.language = language
        self.use = use
        self.value = value

    @classmethod
    def from_dict(cls, data: dict) -> "CodeSystem":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "CodeSystem":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class Property(FhirBaseModel):
    """A property value for this concept.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param str code: Reference to CodeSystem.property.code
    :param str valueCode: Value of the property for this concept
    :param Coding valueCoding: Value of the property for this concept
    :param str valueString: Value of the property for this concept
    :param int valueInteger: Value of the property for this concept
    :param bool valueBoolean: Value of the property for this concept
    :param str valueDateTime: Value of the property for this concept
    :param float valueDecimal: Value of the property for this concept
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "valueCoding": {"class_name": "Coding", "is_contained": False},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        code: "str" = None,
        valueCode: "str" = None,
        valueCoding: "Coding" = None,
        valueString: "str" = None,
        valueInteger: "int" = None,
        valueBoolean: "bool" = None,
        valueDateTime: "str" = None,
        valueDecimal: "float" = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.code = code
        self.valueCode = valueCode
        self.valueCoding = valueCoding
        self.valueString = valueString
        self.valueInteger = valueInteger
        self.valueBoolean = valueBoolean
        self.valueDateTime = valueDateTime
        self.valueDecimal = valueDecimal

    @classmethod
    def from_dict(cls, data: dict) -> "CodeSystem":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "CodeSystem":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class Concept(FhirBaseModel):
    """Concepts that are in the code system. The concept definitions are inherently hierarchical, but the definitions must be consulted to determine what the meanings of the hierarchical relationships are.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param str code: Code that identifies concept
    :param str display: Text to display to the user
    :param str definition: Formal definition
    :param Designation designation: Additional representations for the concept
    :param Property property: Property value for the concept
    :param Concept concept: Child Concepts (is-a/contains/categorizes)
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "designation": {"class_name": "Designation", "is_contained": True},
        "property": {"class_name": "Property", "is_contained": True},
        "concept": {"class_name": "Concept", "is_contained": True},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        code: "str" = None,
        display: "str" = None,
        definition: "str" = None,
        designation: list["Designation"] = None,
        property: list["Property"] = None,
        concept: list["Concept"] = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.code = code
        self.display = display
        self.definition = definition
        self.designation = designation or []
        self.property = property or []
        self.concept = concept or []

    @classmethod
    def from_dict(cls, data: dict) -> "CodeSystem":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "CodeSystem":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class CodeSystem(DomainResource):
    """The CodeSystem resource is used to declare the existence of and describe a code system or code system supplement and its key properties, and optionally define a part or all of its content.
    :param str id: Logical id of this artifact
    :param Meta meta: Metadata about the resource
    :param str implicitRules: A set of rules under which this content was created
    :param str language: Language of the resource content
    :param Narrative text: Text summary of the resource, for human interpretation
    :param Resource contained: Contained, inline Resources
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored
    :param str url: Canonical identifier for this code system, represented as a URI (globally unique) (Coding.system)
    :param Identifier identifier: Additional identifier for the code system (business identifier)
    :param str version: Business version of the code system (Coding.version)
    :param str name: Name for this code system (computer friendly)
    :param str title: Name for this code system (human friendly)
    :param str status: draft | active | retired | unknown
    :param bool experimental: For testing purposes, not real usage
    :param str date: Date last changed
    :param str publisher: Name of the publisher (organization or individual)
    :param ContactDetail contact: Contact details for the publisher
    :param str description: Natural language description of the code system
    :param UsageContext useContext: The context that the content is intended to support
    :param CodeableConcept jurisdiction: Intended jurisdiction for code system (if applicable)
    :param str purpose: Why this code system is defined
    :param str copyright: Use and/or publishing restrictions
    :param bool caseSensitive: If code comparison is case sensitive
    :param str valueSet: Canonical reference to the value set with entire code system
    :param str hierarchyMeaning: grouped-by | is-a | part-of | classified-with
    :param bool compositional: If code system defines a compositional grammar
    :param bool versionNeeded: If definitions are not stable
    :param str content: not-present | example | fragment | complete | supplement
    :param str supplements: Canonical URL of Code System this adds designations and properties to
    :param int count: Total concepts in the code system
    :param Filter filter: Filter that can be used in a value set
    :param Property property: Additional information supplied about each concept
    :param Concept concept: Concepts in the code system
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "meta": {"class_name": "Meta", "is_contained": False},
        "text": {"class_name": "Narrative", "is_contained": False},
        "contained": {"class_name": "Resource", "is_contained": False},
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "identifier": {"class_name": "Identifier", "is_contained": False},
        "contact": {"class_name": "ContactDetail", "is_contained": False},
        "useContext": {"class_name": "UsageContext", "is_contained": False},
        "jurisdiction": {"class_name": "CodeableConcept", "is_contained": False},
        "filter": {"class_name": "Filter", "is_contained": True},
        "property": {"class_name": "Property", "is_contained": True},
        "concept": {"class_name": "Concept", "is_contained": True},
    }

    def __init__(
        self,
        id: "str" = None,
        meta: "Meta" = None,
        implicitRules: "str" = None,
        language: "str" = None,
        text: "Narrative" = None,
        contained: list["Resource"] = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        url: "str" = None,
        identifier: list["Identifier"] = None,
        version: "str" = None,
        name: "str" = None,
        title: "str" = None,
        status: "str" = None,
        experimental: "bool" = None,
        date: "str" = None,
        publisher: "str" = None,
        contact: list["ContactDetail"] = None,
        description: "str" = None,
        useContext: list["UsageContext"] = None,
        jurisdiction: list["CodeableConcept"] = None,
        purpose: "str" = None,
        copyright: "str" = None,
        caseSensitive: "bool" = None,
        valueSet: "str" = None,
        hierarchyMeaning: "str" = None,
        compositional: "bool" = None,
        versionNeeded: "bool" = None,
        content: "str" = None,
        supplements: "str" = None,
        count: "int" = None,
        filter: list["Filter"] = None,
        property: list["Property"] = None,
        concept: list["Concept"] = None,
    ):

        self.resourceType = "CodeSystem"

        self.id = id
        self.meta = meta
        self.implicitRules = implicitRules
        self.language = language
        self.text = text
        self.contained = contained or []
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.url = url
        self.identifier = identifier or []
        self.version = version
        self.name = name
        self.title = title
        self.status = status
        self.experimental = experimental
        self.date = date
        self.publisher = publisher
        self.contact = contact or []
        self.description = description
        self.useContext = useContext or []
        self.jurisdiction = jurisdiction or []
        self.purpose = purpose
        self.copyright = copyright
        self.caseSensitive = caseSensitive
        self.valueSet = valueSet
        self.hierarchyMeaning = hierarchyMeaning
        self.compositional = compositional
        self.versionNeeded = versionNeeded
        self.content = content
        self.supplements = supplements
        self.count = count
        self.filter = filter or []
        self.property = property or []
        self.concept = concept or []

    @classmethod
    def from_dict(cls, data: dict) -> "CodeSystem":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "CodeSystem":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()



"""
Generated class for CoverageEligibilityResponse. 
Time: 2024-06-14 18:37:55
"""

from fhirmodels.R5.BackboneElement import *
from fhirmodels.R5.CodeableConcept import *
from fhirmodels.R5.DomainResource import *
from fhirmodels.R5.Extension import *
from fhirmodels.R5.Identifier import *
from fhirmodels.R5.Meta import *
from fhirmodels.R5.Money import *
from fhirmodels.R5.Narrative import *
from fhirmodels.R5.Period import *
from fhirmodels.R5.Reference import *
from fhirmodels.R5.Resource import *


class Event(FhirBaseModel):
    """Information code for an event with a corresponding date or period.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param CodeableConcept type: Specific event
    :param str whenDateTime: Occurance date or period
    :param Period whenPeriod: Occurance date or period
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "type": {"class_name": "CodeableConcept", "is_contained": False},
        "whenPeriod": {"class_name": "Period", "is_contained": False},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        type: "CodeableConcept" = None,
        whenDateTime: "str" = None,
        whenPeriod: "Period" = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.type = type
        self.whenDateTime = whenDateTime
        self.whenPeriod = whenPeriod

    @classmethod
    def from_dict(cls, data: dict) -> "CoverageEligibilityResponse":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "CoverageEligibilityResponse":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class Benefit(FhirBaseModel):
    """Benefits used to date.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param CodeableConcept type: Benefit classification
    :param int allowedUnsignedInt: Benefits allowed
    :param str allowedString: Benefits allowed
    :param Money allowedMoney: Benefits allowed
    :param int usedUnsignedInt: Benefits used
    :param str usedString: Benefits used
    :param Money usedMoney: Benefits used
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "type": {"class_name": "CodeableConcept", "is_contained": False},
        "allowedMoney": {"class_name": "Money", "is_contained": False},
        "usedMoney": {"class_name": "Money", "is_contained": False},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        type: "CodeableConcept" = None,
        allowedUnsignedInt: "int" = None,
        allowedString: "str" = None,
        allowedMoney: "Money" = None,
        usedUnsignedInt: "int" = None,
        usedString: "str" = None,
        usedMoney: "Money" = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.type = type
        self.allowedUnsignedInt = allowedUnsignedInt
        self.allowedString = allowedString
        self.allowedMoney = allowedMoney
        self.usedUnsignedInt = usedUnsignedInt
        self.usedString = usedString
        self.usedMoney = usedMoney

    @classmethod
    def from_dict(cls, data: dict) -> "CoverageEligibilityResponse":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "CoverageEligibilityResponse":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class Item(FhirBaseModel):
    """Benefits and optionally current balances, and authorization details by category or service.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param CodeableConcept category: Benefit classification
    :param CodeableConcept productOrService: Billing, service, product, or drug code
    :param CodeableConcept modifier: Product or service billing modifiers
    :param Reference provider: Performing practitioner
    :param bool excluded: Excluded from the plan
    :param str name: Short name for the benefit
    :param str description: Description of the benefit or services covered
    :param CodeableConcept network: In or out of network
    :param CodeableConcept unit: Individual or family
    :param CodeableConcept term: Annual or lifetime
    :param Benefit benefit: Benefit Summary
    :param bool authorizationRequired: Authorization required flag
    :param CodeableConcept authorizationSupporting: Type of required supporting materials
    :param str authorizationUrl: Preauthorization requirements endpoint
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "category": {"class_name": "CodeableConcept", "is_contained": False},
        "productOrService": {"class_name": "CodeableConcept", "is_contained": False},
        "modifier": {"class_name": "CodeableConcept", "is_contained": False},
        "provider": {"class_name": "Reference", "is_contained": False},
        "network": {"class_name": "CodeableConcept", "is_contained": False},
        "unit": {"class_name": "CodeableConcept", "is_contained": False},
        "term": {"class_name": "CodeableConcept", "is_contained": False},
        "benefit": {"class_name": "Benefit", "is_contained": True},
        "authorizationSupporting": {
            "class_name": "CodeableConcept",
            "is_contained": False,
        },
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        category: "CodeableConcept" = None,
        productOrService: "CodeableConcept" = None,
        modifier: list["CodeableConcept"] = None,
        provider: "Reference" = None,
        excluded: "bool" = None,
        name: "str" = None,
        description: "str" = None,
        network: "CodeableConcept" = None,
        unit: "CodeableConcept" = None,
        term: "CodeableConcept" = None,
        benefit: list["Benefit"] = None,
        authorizationRequired: "bool" = None,
        authorizationSupporting: list["CodeableConcept"] = None,
        authorizationUrl: "str" = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.category = category
        self.productOrService = productOrService
        self.modifier = modifier or []
        self.provider = provider
        self.excluded = excluded
        self.name = name
        self.description = description
        self.network = network
        self.unit = unit
        self.term = term
        self.benefit = benefit or []
        self.authorizationRequired = authorizationRequired
        self.authorizationSupporting = authorizationSupporting or []
        self.authorizationUrl = authorizationUrl

    @classmethod
    def from_dict(cls, data: dict) -> "CoverageEligibilityResponse":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "CoverageEligibilityResponse":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class Insurance(FhirBaseModel):
    """Financial instruments for reimbursement for the health care products and services.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param Reference coverage: Insurance information
    :param bool inforce: Coverage inforce indicator
    :param Period benefitPeriod: When the benefits are applicable
    :param Item item: Benefits and authorization details
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "coverage": {"class_name": "Reference", "is_contained": False},
        "benefitPeriod": {"class_name": "Period", "is_contained": False},
        "item": {"class_name": "Item", "is_contained": True},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        coverage: "Reference" = None,
        inforce: "bool" = None,
        benefitPeriod: "Period" = None,
        item: list["Item"] = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.coverage = coverage
        self.inforce = inforce
        self.benefitPeriod = benefitPeriod
        self.item = item or []

    @classmethod
    def from_dict(cls, data: dict) -> "CoverageEligibilityResponse":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "CoverageEligibilityResponse":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class Error(FhirBaseModel):
    """Errors encountered during the processing of the request.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param CodeableConcept code: Error code detailing processing issues
    :param str expression: FHIRPath of element(s) related to issue
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "code": {"class_name": "CodeableConcept", "is_contained": False},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        code: "CodeableConcept" = None,
        expression: list["str"] = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.code = code
        self.expression = expression or []

    @classmethod
    def from_dict(cls, data: dict) -> "CoverageEligibilityResponse":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "CoverageEligibilityResponse":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class CoverageEligibilityResponse(DomainResource):
    """This resource provides eligibility and plan details from the processing of an CoverageEligibilityRequest resource.
    :param str id: Logical id of this artifact
    :param Meta meta: Metadata about the resource
    :param str implicitRules: A set of rules under which this content was created
    :param str language: Language of the resource content
    :param Narrative text: Text summary of the resource, for human interpretation
    :param Resource contained: Contained, inline Resources
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored
    :param Identifier identifier: Business Identifier for coverage eligiblity request
    :param str status: active | cancelled | draft | entered-in-error
    :param str purpose: auth-requirements | benefits | discovery | validation
    :param Reference patient: Intended recipient of products and services
    :param Event event: Event information
    :param str servicedDate: Estimated date or dates of service
    :param Period servicedPeriod: Estimated date or dates of service
    :param str created: Response creation date
    :param Reference requestor: Party responsible for the request
    :param Reference request: Eligibility request reference
    :param str outcome: queued | complete | error | partial
    :param str disposition: Disposition Message
    :param Reference insurer: Coverage issuer
    :param Insurance insurance: Patient insurance information
    :param str preAuthRef: Preauthorization reference
    :param CodeableConcept form: Printed form identifier
    :param Error error: Processing errors
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "meta": {"class_name": "Meta", "is_contained": False},
        "text": {"class_name": "Narrative", "is_contained": False},
        "contained": {"class_name": "Resource", "is_contained": False},
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "identifier": {"class_name": "Identifier", "is_contained": False},
        "patient": {"class_name": "Reference", "is_contained": False},
        "event": {"class_name": "Event", "is_contained": True},
        "servicedPeriod": {"class_name": "Period", "is_contained": False},
        "requestor": {"class_name": "Reference", "is_contained": False},
        "request": {"class_name": "Reference", "is_contained": False},
        "insurer": {"class_name": "Reference", "is_contained": False},
        "insurance": {"class_name": "Insurance", "is_contained": True},
        "form": {"class_name": "CodeableConcept", "is_contained": False},
        "error": {"class_name": "Error", "is_contained": True},
    }

    def __init__(
        self,
        id: "str" = None,
        meta: "Meta" = None,
        implicitRules: "str" = None,
        language: "str" = None,
        text: "Narrative" = None,
        contained: list["Resource"] = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        identifier: list["Identifier"] = None,
        status: "str" = None,
        purpose: list["str"] = None,
        patient: "Reference" = None,
        event: list["Event"] = None,
        servicedDate: "str" = None,
        servicedPeriod: "Period" = None,
        created: "str" = None,
        requestor: "Reference" = None,
        request: "Reference" = None,
        outcome: "str" = None,
        disposition: "str" = None,
        insurer: "Reference" = None,
        insurance: list["Insurance"] = None,
        preAuthRef: "str" = None,
        form: "CodeableConcept" = None,
        error: list["Error"] = None,
    ):

        self.resourceType = "CoverageEligibilityResponse"

        self.id = id
        self.meta = meta
        self.implicitRules = implicitRules
        self.language = language
        self.text = text
        self.contained = contained or []
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.identifier = identifier or []
        self.status = status
        self.purpose = purpose or []
        self.patient = patient
        self.event = event or []
        self.servicedDate = servicedDate
        self.servicedPeriod = servicedPeriod
        self.created = created
        self.requestor = requestor
        self.request = request
        self.outcome = outcome
        self.disposition = disposition
        self.insurer = insurer
        self.insurance = insurance or []
        self.preAuthRef = preAuthRef
        self.form = form
        self.error = error or []

    @classmethod
    def from_dict(cls, data: dict) -> "CoverageEligibilityResponse":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "CoverageEligibilityResponse":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()



"""
Generated class for Organization. 
Time: 2024-06-14 18:37:55
"""

from fhirmodels.R5.BackboneElement import *
from fhirmodels.R5.CodeableConcept import *
from fhirmodels.R5.DomainResource import *
from fhirmodels.R5.ExtendedContactDetail import *
from fhirmodels.R5.Extension import *
from fhirmodels.R5.Identifier import *
from fhirmodels.R5.Meta import *
from fhirmodels.R5.Narrative import *
from fhirmodels.R5.Period import *
from fhirmodels.R5.Reference import *
from fhirmodels.R5.Resource import *


class Qualification(FhirBaseModel):
    """The official certifications, accreditations, training, designations and licenses that authorize and/or otherwise endorse the provision of care by the organization.

    For example, an approval to provide a type of services issued by a certifying body (such as the US Joint Commission) to an organization.:param str id: Unique id for inter-element referencing
        :param Extension extension: Additional content defined by implementations
        :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
        :param Identifier identifier: An identifier for this qualification for the organization
        :param CodeableConcept code: Coded representation of the qualification
        :param Period period: Period during which the qualification is valid
        :param Reference issuer: Organization that regulates and issues the qualification
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "identifier": {"class_name": "Identifier", "is_contained": False},
        "code": {"class_name": "CodeableConcept", "is_contained": False},
        "period": {"class_name": "Period", "is_contained": False},
        "issuer": {"class_name": "Reference", "is_contained": False},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        identifier: list["Identifier"] = None,
        code: "CodeableConcept" = None,
        period: "Period" = None,
        issuer: "Reference" = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.identifier = identifier or []
        self.code = code
        self.period = period
        self.issuer = issuer

    @classmethod
    def from_dict(cls, data: dict) -> "Organization":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "Organization":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class Organization(DomainResource):
    """A formally or informally recognized grouping of people or organizations formed for the purpose of achieving some form of collective action.  Includes companies, institutions, corporations, departments, community groups, healthcare practice groups, payer/insurer, etc.
    :param str id: Logical id of this artifact
    :param Meta meta: Metadata about the resource
    :param str implicitRules: A set of rules under which this content was created
    :param str language: Language of the resource content
    :param Narrative text: Text summary of the resource, for human interpretation
    :param Resource contained: Contained, inline Resources
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored
    :param Identifier identifier: Identifies this organization  across multiple systems
    :param bool active: Whether the organization's record is still in active use
    :param CodeableConcept type: Kind of organization
    :param str name: Name used for the organization
    :param str alias: A list of alternate names that the organization is known as, or was known as in the past
    :param str description: Additional details about the Organization that could be displayed as further information to identify the Organization beyond its name
    :param ExtendedContactDetail contact: Official contact details for the Organization
    :param Reference partOf: The organization of which this organization forms a part
    :param Reference endpoint: Technical endpoints providing access to services operated for the organization
    :param Qualification qualification: Qualifications, certifications, accreditations, licenses, training, etc. pertaining to the provision of care
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "meta": {"class_name": "Meta", "is_contained": False},
        "text": {"class_name": "Narrative", "is_contained": False},
        "contained": {"class_name": "Resource", "is_contained": False},
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "identifier": {"class_name": "Identifier", "is_contained": False},
        "type": {"class_name": "CodeableConcept", "is_contained": False},
        "contact": {"class_name": "ExtendedContactDetail", "is_contained": False},
        "partOf": {"class_name": "Reference", "is_contained": False},
        "endpoint": {"class_name": "Reference", "is_contained": False},
        "qualification": {"class_name": "Qualification", "is_contained": True},
    }

    def __init__(
        self,
        id: "str" = None,
        meta: "Meta" = None,
        implicitRules: "str" = None,
        language: "str" = None,
        text: "Narrative" = None,
        contained: list["Resource"] = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        identifier: list["Identifier"] = None,
        active: "bool" = None,
        type: list["CodeableConcept"] = None,
        name: "str" = None,
        alias: list["str"] = None,
        description: "str" = None,
        contact: list["ExtendedContactDetail"] = None,
        partOf: "Reference" = None,
        endpoint: list["Reference"] = None,
        qualification: list["Qualification"] = None,
    ):

        self.resourceType = "Organization"

        self.id = id
        self.meta = meta
        self.implicitRules = implicitRules
        self.language = language
        self.text = text
        self.contained = contained or []
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.identifier = identifier or []
        self.active = active
        self.type = type or []
        self.name = name
        self.alias = alias or []
        self.description = description
        self.contact = contact or []
        self.partOf = partOf
        self.endpoint = endpoint or []
        self.qualification = qualification or []

    @classmethod
    def from_dict(cls, data: dict) -> "Organization":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "Organization":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()



"""
Generated class for Consent. 
Time: 2024-06-14 18:37:17
"""

from fhirmodels.R4.Attachment import *
from fhirmodels.R4.BackboneElement import *
from fhirmodels.R4.CodeableConcept import *
from fhirmodels.R4.Coding import *
from fhirmodels.R4.DomainResource import *
from fhirmodels.R4.Extension import *
from fhirmodels.R4.Identifier import *
from fhirmodels.R4.Meta import *
from fhirmodels.R4.Narrative import *
from fhirmodels.R4.Period import *
from fhirmodels.R4.Reference import *
from fhirmodels.R4.Resource import *


class Policy(FhirBaseModel):
    """The references to the policies that are included in this consent scope. Policies may be organizational, but are often defined jurisdictionally, or in law.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param str authority: Enforcement source for policy
    :param str uri: Specific policy covered by this consent
    :param CodeableConcept policyRule: Regulation that this consents to
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "policyRule": {"class_name": "CodeableConcept", "is_contained": False},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        authority: "str" = None,
        uri: "str" = None,
        policyRule: "CodeableConcept" = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.authority = authority
        self.uri = uri
        self.policyRule = policyRule

    @classmethod
    def from_dict(cls, data: dict) -> "Consent":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "Consent":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class Verification(FhirBaseModel):
    """Whether a treatment instruction (e.g. artificial respiration yes or no) was verified with the patient, his/her family or another authorized person.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param bool verified: Has been verified
    :param Reference verifiedWith: Person who verified
    :param str verificationDate: When consent verified
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "verifiedWith": {"class_name": "Reference", "is_contained": False},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        verified: "bool" = None,
        verifiedWith: "Reference" = None,
        verificationDate: "str" = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.verified = verified
        self.verifiedWith = verifiedWith
        self.verificationDate = verificationDate

    @classmethod
    def from_dict(cls, data: dict) -> "Consent":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "Consent":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class Actor(FhirBaseModel):
    """Who or what is controlled by this rule. Use group to identify a set of actors by some property they share (e.g. 'admitting officers').:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param CodeableConcept role: How the actor is involved
    :param Reference reference: Resource for the actor (or group, by role)
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "role": {"class_name": "CodeableConcept", "is_contained": False},
        "reference": {"class_name": "Reference", "is_contained": False},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        role: "CodeableConcept" = None,
        reference: "Reference" = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.role = role
        self.reference = reference

    @classmethod
    def from_dict(cls, data: dict) -> "Consent":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "Consent":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class Data(FhirBaseModel):
    """The resources controlled by this rule if specific resources are referenced.:param Period dataPeriod: Timeframe for data controlled by this rule
    :param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param str meaning: instance | related | dependents | authoredby
    :param Reference reference: The actual data reference
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "dataPeriod": {"class_name": "Period", "is_contained": False},
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "reference": {"class_name": "Reference", "is_contained": False},
    }

    def __init__(
        self,
        dataPeriod: "Period" = None,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        meaning: "str" = None,
        reference: "Reference" = None,
    ):
        self.dataPeriod = dataPeriod
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.meaning = meaning
        self.reference = reference

    @classmethod
    def from_dict(cls, data: dict) -> "Consent":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "Consent":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class Provision(FhirBaseModel):
    """An exception to the base policy of this consent. An exception can be an addition or removal of access permissions.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param str type: deny | permit
    :param Period period: Timeframe for this rule
    :param Actor actor: Who|what controlled by this rule (or group, by role)
    :param CodeableConcept action: Actions controlled by this rule
    :param Coding securityLabel: Security Labels that define affected resources
    :param Coding purpose: Context of activities covered by this rule
    :param Coding _class: e.g. Resource Type, Profile, CDA, etc.
    :param CodeableConcept code: e.g. LOINC or SNOMED CT code, etc. in the content
    :param Period dataPeriod: Timeframe for data controlled by this rule
    :param Data data: Data controlled by this rule
    :param Provision provision: Nested Exception Rules
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "period": {"class_name": "Period", "is_contained": False},
        "actor": {"class_name": "Actor", "is_contained": True},
        "action": {"class_name": "CodeableConcept", "is_contained": False},
        "securityLabel": {"class_name": "Coding", "is_contained": False},
        "purpose": {"class_name": "Coding", "is_contained": False},
        "_class": {"class_name": "Coding", "is_contained": False},
        "code": {"class_name": "CodeableConcept", "is_contained": False},
        "dataPeriod": {"class_name": "Period", "is_contained": False},
        "data": {"class_name": "Data", "is_contained": True},
        "provision": {"class_name": "Provision", "is_contained": True},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        type: "str" = None,
        period: "Period" = None,
        actor: list["Actor"] = None,
        action: list["CodeableConcept"] = None,
        securityLabel: list["Coding"] = None,
        purpose: list["Coding"] = None,
        _class: list["Coding"] = None,
        code: list["CodeableConcept"] = None,
        dataPeriod: "Period" = None,
        data: list["Data"] = None,
        provision: list["Provision"] = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.type = type
        self.period = period
        self.actor = actor or []
        self.action = action or []
        self.securityLabel = securityLabel or []
        self.purpose = purpose or []
        self._class = _class or []
        self.code = code or []
        self.dataPeriod = dataPeriod
        self.data = data or []
        self.provision = provision or []

    @classmethod
    def from_dict(cls, data: dict) -> "Consent":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "Consent":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class Consent(DomainResource):
    """A record of a healthcare consumer’s  choices, which permits or denies identified recipient(s) or recipient role(s) to perform one or more actions within a given policy context, for specific purposes and periods of time.
    :param str id: Logical id of this artifact
    :param Meta meta: Metadata about the resource
    :param str implicitRules: A set of rules under which this content was created
    :param str language: Language of the resource content
    :param Narrative text: Text summary of the resource, for human interpretation
    :param Resource contained: Contained, inline Resources
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored
    :param Identifier identifier: Identifier for this record (external references)
    :param str status: draft | proposed | active | rejected | inactive | entered-in-error
    :param CodeableConcept scope: Which of the four areas this resource covers (extensible)
    :param CodeableConcept category: Classification of the consent statement - for indexing/retrieval
    :param Reference patient: Who the consent applies to
    :param str dateTime: When this Consent was created or indexed
    :param Reference performer: Who is agreeing to the policy and rules
    :param Reference organization: Custodian of the consent
    :param Attachment sourceAttachment: Source from which this consent is taken
    :param Reference sourceReference: Source from which this consent is taken
    :param Policy policy: Policies covered by this consent
    :param Verification verification: Consent Verified by patient or family
    :param Provision provision: Constraints to the base Consent.policyRule
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "meta": {"class_name": "Meta", "is_contained": False},
        "text": {"class_name": "Narrative", "is_contained": False},
        "contained": {"class_name": "Resource", "is_contained": False},
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "identifier": {"class_name": "Identifier", "is_contained": False},
        "scope": {"class_name": "CodeableConcept", "is_contained": False},
        "category": {"class_name": "CodeableConcept", "is_contained": False},
        "patient": {"class_name": "Reference", "is_contained": False},
        "performer": {"class_name": "Reference", "is_contained": False},
        "organization": {"class_name": "Reference", "is_contained": False},
        "sourceAttachment": {"class_name": "Attachment", "is_contained": False},
        "sourceReference": {"class_name": "Reference", "is_contained": False},
        "policy": {"class_name": "Policy", "is_contained": True},
        "verification": {"class_name": "Verification", "is_contained": True},
        "provision": {"class_name": "Provision", "is_contained": True},
    }

    def __init__(
        self,
        id: "str" = None,
        meta: "Meta" = None,
        implicitRules: "str" = None,
        language: "str" = None,
        text: "Narrative" = None,
        contained: list["Resource"] = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        identifier: list["Identifier"] = None,
        status: "str" = None,
        scope: "CodeableConcept" = None,
        category: list["CodeableConcept"] = None,
        patient: "Reference" = None,
        dateTime: "str" = None,
        performer: list["Reference"] = None,
        organization: list["Reference"] = None,
        sourceAttachment: "Attachment" = None,
        sourceReference: "Reference" = None,
        policy: list["Policy"] = None,
        verification: list["Verification"] = None,
        provision: "Provision" = None,
    ):

        self.resourceType = "Consent"

        self.id = id
        self.meta = meta
        self.implicitRules = implicitRules
        self.language = language
        self.text = text
        self.contained = contained or []
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.identifier = identifier or []
        self.status = status
        self.scope = scope
        self.category = category or []
        self.patient = patient
        self.dateTime = dateTime
        self.performer = performer or []
        self.organization = organization or []
        self.sourceAttachment = sourceAttachment
        self.sourceReference = sourceReference
        self.policy = policy or []
        self.verification = verification or []
        self.provision = provision

    @classmethod
    def from_dict(cls, data: dict) -> "Consent":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "Consent":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()



"""
Generated class for PaymentReconciliation. 
Time: 2024-06-14 18:37:49
"""

from fhirmodels.R4B.BackboneElement import *
from fhirmodels.R4B.CodeableConcept import *
from fhirmodels.R4B.DomainResource import *
from fhirmodels.R4B.Extension import *
from fhirmodels.R4B.Identifier import *
from fhirmodels.R4B.Meta import *
from fhirmodels.R4B.Money import *
from fhirmodels.R4B.Narrative import *
from fhirmodels.R4B.Period import *
from fhirmodels.R4B.Reference import *
from fhirmodels.R4B.Resource import *


class Detail(FhirBaseModel):
    """Distribution of the payment amount for a previously acknowledged payable.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param Identifier identifier: Business identifier of the payment detail
    :param Identifier predecessor: Business identifier of the prior payment detail
    :param CodeableConcept type: Category of payment
    :param Reference request: Request giving rise to the payment
    :param Reference submitter: Submitter of the request
    :param Reference response: Response committing to a payment
    :param str date: Date of commitment to pay
    :param Reference responsible: Contact for the response
    :param Reference payee: Recipient of the payment
    :param Money amount: Amount allocated to this payable
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "identifier": {"class_name": "Identifier", "is_contained": False},
        "predecessor": {"class_name": "Identifier", "is_contained": False},
        "type": {"class_name": "CodeableConcept", "is_contained": False},
        "request": {"class_name": "Reference", "is_contained": False},
        "submitter": {"class_name": "Reference", "is_contained": False},
        "response": {"class_name": "Reference", "is_contained": False},
        "responsible": {"class_name": "Reference", "is_contained": False},
        "payee": {"class_name": "Reference", "is_contained": False},
        "amount": {"class_name": "Money", "is_contained": False},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        identifier: "Identifier" = None,
        predecessor: "Identifier" = None,
        type: "CodeableConcept" = None,
        request: "Reference" = None,
        submitter: "Reference" = None,
        response: "Reference" = None,
        date: "str" = None,
        responsible: "Reference" = None,
        payee: "Reference" = None,
        amount: "Money" = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.identifier = identifier
        self.predecessor = predecessor
        self.type = type
        self.request = request
        self.submitter = submitter
        self.response = response
        self.date = date
        self.responsible = responsible
        self.payee = payee
        self.amount = amount

    @classmethod
    def from_dict(cls, data: dict) -> "PaymentReconciliation":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "PaymentReconciliation":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class ProcessNote(FhirBaseModel):
    """A note that describes or explains the processing in a human readable form.:param str id: Unique id for inter-element referencing
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored even if unrecognized
    :param str type: display | print | printoper
    :param str text: Note explanatory text
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
    }

    def __init__(
        self,
        id: "str" = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        type: "str" = None,
        text: "str" = None,
    ):
        self.id = id
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.type = type
        self.text = text

    @classmethod
    def from_dict(cls, data: dict) -> "PaymentReconciliation":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "PaymentReconciliation":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()


class PaymentReconciliation(DomainResource):
    """This resource provides the details including amount of a payment and allocates the payment items being paid.
    :param str id: Logical id of this artifact
    :param Meta meta: Metadata about the resource
    :param str implicitRules: A set of rules under which this content was created
    :param str language: Language of the resource content
    :param Narrative text: Text summary of the resource, for human interpretation
    :param Resource contained: Contained, inline Resources
    :param Extension extension: Additional content defined by implementations
    :param Extension modifierExtension: Extensions that cannot be ignored
    :param Identifier identifier: Business Identifier for a payment reconciliation
    :param str status: active | cancelled | draft | entered-in-error
    :param Period period: Period covered
    :param str created: Creation date
    :param Reference paymentIssuer: Party generating payment
    :param Reference request: Reference to requesting resource
    :param Reference requestor: Responsible practitioner
    :param str outcome: queued | complete | error | partial
    :param str disposition: Disposition message
    :param str paymentDate: When payment issued
    :param Money paymentAmount: Total amount of Payment
    :param Identifier paymentIdentifier: Business identifier for the payment
    :param Detail detail: Settlement particulars
    :param CodeableConcept formCode: Printed form identifier
    :param ProcessNote processNote: Note concerning processing
    """

    # needed for complex properties where the element name is different from the class name
    property_class_info = {
        "meta": {"class_name": "Meta", "is_contained": False},
        "text": {"class_name": "Narrative", "is_contained": False},
        "contained": {"class_name": "Resource", "is_contained": False},
        "extension": {"class_name": "Extension", "is_contained": False},
        "modifierExtension": {"class_name": "Extension", "is_contained": False},
        "identifier": {"class_name": "Identifier", "is_contained": False},
        "period": {"class_name": "Period", "is_contained": False},
        "paymentIssuer": {"class_name": "Reference", "is_contained": False},
        "request": {"class_name": "Reference", "is_contained": False},
        "requestor": {"class_name": "Reference", "is_contained": False},
        "paymentAmount": {"class_name": "Money", "is_contained": False},
        "paymentIdentifier": {"class_name": "Identifier", "is_contained": False},
        "detail": {"class_name": "Detail", "is_contained": True},
        "formCode": {"class_name": "CodeableConcept", "is_contained": False},
        "processNote": {"class_name": "ProcessNote", "is_contained": True},
    }

    def __init__(
        self,
        id: "str" = None,
        meta: "Meta" = None,
        implicitRules: "str" = None,
        language: "str" = None,
        text: "Narrative" = None,
        contained: list["Resource"] = None,
        extension: list["Extension"] = None,
        modifierExtension: list["Extension"] = None,
        identifier: list["Identifier"] = None,
        status: "str" = None,
        period: "Period" = None,
        created: "str" = None,
        paymentIssuer: "Reference" = None,
        request: "Reference" = None,
        requestor: "Reference" = None,
        outcome: "str" = None,
        disposition: "str" = None,
        paymentDate: "str" = None,
        paymentAmount: "Money" = None,
        paymentIdentifier: "Identifier" = None,
        detail: list["Detail"] = None,
        formCode: "CodeableConcept" = None,
        processNote: list["ProcessNote"] = None,
    ):

        self.resourceType = "PaymentReconciliation"

        self.id = id
        self.meta = meta
        self.implicitRules = implicitRules
        self.language = language
        self.text = text
        self.contained = contained or []
        self.extension = extension or []
        self.modifierExtension = modifierExtension or []
        self.identifier = identifier or []
        self.status = status
        self.period = period
        self.created = created
        self.paymentIssuer = paymentIssuer
        self.request = request
        self.requestor = requestor
        self.outcome = outcome
        self.disposition = disposition
        self.paymentDate = paymentDate
        self.paymentAmount = paymentAmount
        self.paymentIdentifier = paymentIdentifier
        self.detail = detail or []
        self.formCode = formCode
        self.processNote = processNote or []

    @classmethod
    def from_dict(cls, data: dict) -> "PaymentReconciliation":
        return super().from_dict(data)

    @classmethod
    def from_obj(self, obj: object) -> "PaymentReconciliation":
        return super().from_obj(obj)

    def as_dict(self) -> dict:
        return super().as_dict()
